-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftCreatureTypeData")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Save Creature Type Data"] = true,
	["Stores creature type information between sessions"] = true,
	
	-- short forms
	["Beast"]			= true,
	["Demon"]			= true,
	["Dragonkin"]		= true,
	["Humanoid"]		= true,
	["Elemental"]		= true,
	["Undead"]			= true,

	-- This is checked against when we gather data. If it's not specified, don't store it.
	["Not specified"]	= true,

	["ShortBeast"]		= "B",
	["ShortDemon"]		= "D",
	["ShortDragonkin"]	= "K",
	["ShortHumanoid"]	= "H",
	["ShortElemental"]	= "E",
	["ShortUndead"]		= "U",
} end)

-----------------------------------------------------------------------------

local creatureTypeData = { }
local createTypeTextLookup =
{
	[L["Beast"]]		= L["ShortBeast"],
	[L["Demon"]]		= L["ShortDemon"],
	[L["Dragonkin"]]	= L["ShortDragonkin"],
	[L["Elemental"]]	= L["ShortElemental"],
	[L["Humanoid"]]		= L["ShortHumanoid"],
	[L["Undead"]]		= L["ShortUndead"],
}

-----------------------------------------------------------------------------

local AloftCreatureTypeData = Aloft:NewModule("CreatureTypeData", "AceEvent-2.0")
AloftCreatureTypeData.db = Aloft:AcquireDBNamespace("creatureTypeData")
Aloft:RegisterDefaults("creatureTypeData", "account", {	creatureTypeData = { } })
Aloft:RegisterDefaults("creatureTypeData", "profile", { save = true })

-----------------------------------------------------------------------------

Aloft.Options.args.data.args.creatureType =
{
	type = 'toggle',
	name = L["Save Creature Type Data"],
	desc = L["Stores creature type information between sessions"],
	get = function() return AloftCreatureTypeData.db.profile.save end,
	set = function(v)
			AloftCreatureTypeData.db.profile.save = v
			if v then
				AloftCreatureTypeData.db.server.creatureTypeData = creatureTypeData
			else
				AloftCreatureTypeData.db.server.creatureTypeData = { }
			end
		  end,
}

-----------------------------------------------------------------------------

AloftTagDirectAccessData.creatureType = true
AloftTagData.creatureType = { data = "creatureType", events = "Aloft:OnCreatureTypeDataChanged" }

AloftTagMethods.shortCreatureType = function(aloftData, value)
	return value and aloftData.creatureType and createTypeTextLookup[aloftData.creatureType] or nil
end
AloftTagData.shortCreatureType = { data = "creatureType", events = "Aloft:OnCreatureTypeDataChanged" }

function AloftCreatureTypeData:ProvidesData()
	return "creatureType"
end

function AloftCreatureTypeData:EnableDataSource()
    self:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "UpdateCreatureTypeFromMouseOver")
    self:RegisterEvent("PLAYER_TARGET_CHANGED", "UpdateCreatureTypeFromTarget")
end

-----------------------------------------------------------------------------

function AloftCreatureTypeData:OnEnable()
	if self.db.profile.save then
		creatureTypeData = self.db.account.creatureTypeData
	end
end

-----------------------------------------------------------------------------

function AloftCreatureTypeData:UpdateData(aloftData)
	aloftData.creatureType = creatureTypeData[aloftData.name]
end

-----------------------------------------------------------------------------

function AloftCreatureTypeData:UpdateFromUnit(unitid)
	if not UnitExists(unitid) then return end
	if UnitIsPlayer(unitid) then return end
	
	-- Don't store normal unit classifications to reduce the database
	local creatureType = UnitCreatureType(unitid)
	if creatureType == "" or creatureType == L["Not specified"] then return end
	
	local name = UnitName(unitid)
	if creatureType == creatureTypeData[name] then return end
	creatureTypeData[name] = creatureType
	
	-- AloftData uses level as nil for tag purposes if its indeterminate
	for aloftData in Aloft:IterateVisibleNameplates() do
		if aloftData.name == name then
			aloftData.creatureType = creatureType
			self:TriggerEvent("Aloft:OnCreatureTypeDataChanged", aloftData)
		end
	end
end

function AloftCreatureTypeData:UpdateCreatureTypeFromMouseOver()
	self:UpdateFromUnit("mouseover")
end

function AloftCreatureTypeData:UpdateCreatureTypeFromTarget()
	self:UpdateFromUnit("target")
end

-----------------------------------------------------------------------------
